@@ -1,4 +1,4 @@
-from fractions import Fraction
+from __future__ import division
 
 class Block(object):
     """
@@ -19,7 +19,7 @@ class Block(object):
         "slot",
     )
 
-    def __init__(self, slot, name, drop=None, replace=0, ratio=None,
+    def __init__(self, slot, name, drop=None, replace=0, ratio=1,
             quantity=1):
         """
         A block in a chunk.
@@ -36,9 +36,8 @@ class Block(object):
             replace : int
                 The type of block to place in the map when instances of this
                 block are destroyed. Defaults to air.
-            ratio : `Fraction`
-                The probability of this block dropping a block on destruction,
-                or None if this block always drops a block when destroyed.
+            ratio : float
+                The probability of this block dropping a block on destruction.
             quantity : int
                 The number of blocks dropped when this block is destroyed.
         """
@@ -102,11 +101,11 @@ block_names = [
     "leaves",
     "sponge",
     "glass",
-    "red-cloth",
-    "orange-cloth",
-    "yellow-cloth",
-    "lime-cloth",
-    "green-cloth",
+    "lapis-lazuli-ore",
+    "lapis-lazuli",
+    "dispenser",
+    "sandstone",
+    "note-block",
     "aqua-cloth",
     "cyan-cloth",
     "blue-cloth",
@@ -117,7 +116,7 @@ block_names = [
     "pink-cloth",
     "black-cloth",
     "grey-cloth",
-    "white-cloth",
+    "wool",
     "flower",
     "rose",
     "brown-mushroom",
@@ -164,7 +163,7 @@ block_names = [
     "snow-block",
     "cactus",
     "clay",
-    "reed",
+    "sugar-cane",
     "jukebox",
     "fence",
     "pumpkin",
@@ -173,6 +172,7 @@ block_names = [
     "lightstone",
     "portal",
     "jack-o-lantern",
+    "cake",
 ]
 
 item_names = [
@@ -202,7 +202,7 @@ item_names = [
     "diamond-axe",
     "stick",
     "bowl",
-    "mushroom-coup",
+    "mushroom-soup",
     "gold-sword",
     "gold-shovel",
     "gold-pickaxe",
@@ -286,16 +286,17 @@ drops = {}
 
 # Block -> block drops.
 # If the drop block is zero, then it drops nothing.
-drops[1]  = 4  # Stone           -> Cobblestone
-drops[2]  = 3  # Grass           -> Dirt
-drops[18] = 6  # Leaves          -> Sapling
-drops[20] = 0  # Glass
-drops[60] = 3  # Soil            -> Dirt
-drops[62] = 61 # Burning Furnace -> Furnace
-drops[78] = 0  # Snow
-drops[79] = 0  # Ice
+drops[1]  = 4   # Stone           -> Cobblestone
+drops[2]  = 3   # Grass           -> Dirt
+drops[18] = 6   # Leaves          -> Sapling
+drops[20] = 0   # Glass
+drops[60] = 3   # Soil            -> Dirt
+drops[62] = 61  # Burning Furnace -> Furnace
+drops[78] = 0   # Snow
+drops[79] = 0   # Ice
 
 # Block -> item drops.
+drops[13] = 318 # Gravel            -> Flint
 drops[16] = 263 # Coal Ore Block    -> Coal
 drops[56] = 264 # Diamond Ore Block -> Diamond
 drops[63] = 323 # Sign Post         -> Sign Item
@@ -304,6 +305,7 @@ drops[68] = 323 # Wall Sign         -> Sign Item
 drops[71] = 330 # Iron Door         -> Iron Door Item
 drops[73] = 331 # Redstone Ore      -> Redstone Dust
 drops[74] = 331 # Redstone Ore      -> Redstone Dust
+drops[82] = 337 # Clay              -> Clay Balls
 drops[83] = 338 # Reed              -> Reed Item
 drops[89] = 348 # Lightstone        -> Lightstone Dust
 
@@ -313,12 +315,14 @@ replaces[79] = 8 # Ice -> Water
 
 ratios = {}
 
-ratios[18] = Fraction(1, 9) # Leaves
+ratios[13] = 1 / 10 # Gravel
+ratios[18] = 1 / 9  # Leaves
 
 quantities = {}
 
 quantities[73] = 5 # Redstone Ore -> Redstone Dust
 quantities[74] = 5 # Redstone Ore -> Redstone Dust
+quantities[82] = 4 # Clay         -> Clay Balls
 
 blocks = {}
 """
