@@ -1,6 +1,15 @@
-import ConfigParser
+from ConfigParser import SafeConfigParser
 from os.path import expanduser
 
+class BravoConfigParser(SafeConfigParser):
+    """
+    Extended ``ConfigParser``.
+    """
+
+    def getlist(self, section, option, separator=","):
+        s = self.get(section, option)
+        return [i.strip() for i in s.split(separator)]
+
 defaults = {
     "authenticator": "offline",
     "generators": "simplex,erosion,watertable,grass,safety",
@@ -8,10 +17,11 @@ defaults = {
     "dig_hooks": "give,replace,alpha_snow",
     "fancy_console": "true",
     "ampoule": "true",
-    "serializer": "json",
+    "serializer": "nbt",
+    "perm_cache": "yes",
 }
 
-configuration = ConfigParser.SafeConfigParser(defaults)
+configuration = BravoConfigParser(defaults)
 configuration.add_section("bravo")
 
 # XXX improve on this
